{% extends 'core/vue_base.html' %}
{% load i18n static dashboard %}

{% block browser_title %}{% blocktrans %}Dummy page{% endblocktrans %}{% endblock %}
{% block page_title %}{% blocktrans %}Dummy page{% endblocktrans %}{% endblock %}

{% block content %}
  <p>Nothing here</p>
{% endblock %}
