{% extends "core/base.html" %}

{% load i18n material_form account %}

{% block browser_title %}{% trans "Authorize" %}{% endblock %}
{% block page_title %}{% trans "Authorize" %}{% endblock %}

{% block content %}
{% if process == "connect" %}
    <p class="flow-text">
       <i class="material-icons iconify left" data-icon="mdi:information-outline"></i>
       {% blocktrans with provider.name as provider %}You are about to connect a new third party account from {{ provider }}.{% endblocktrans %}
    </p>
    <form method="post">
      {% csrf_token %}
      {% form form=form %}{% endform %}
      {% trans "Confirm" as caption %}
      {% include "core/partials/save_button.html" with caption=caption icon="how_to_reg" %}
    </form>
{% else %}
    <p class="flow-text">
       <i class="material-icons iconify left small" data-icon="mdi:information-outline"></i>
       {% blocktrans with provider.name as provider %}You are about to sign in using a third party account from {{ provider }}.{% endblocktrans %}
    </p>
    <form method="post">
      {% csrf_token %}
      {% form form=form %}{% endform %}
      {% trans "Continue" as caption %}
      {% include "core/partials/save_button.html" with caption=caption icon="how_to_reg" %}
    </form>

{% endif %}
{% endblock %}
