{# -*- engine:django -*- #}

{% for item in FOOTER_MENU.items.all %}
  <a class="blue-text text-lighten-4 btn-flat" href="{{ item.url }}">
    {% if item.icon %}
      <i class="material-icons iconify footer-icon left" data-icon="mdi:{{ item.icon }}"></i>
    {% endif %}
    {{ item.name }}
  </a>
{% endfor %}
