{% extends "core/base.html" %}

{% load i18n %}

{% block browser_title %}{% blocktrans %}The invite feature is disabled{% endblocktrans %}{% endblock %}
{% block no_page_title %}{% endblock %}

{% block content %}
  <div class="container">
    <div class="card red">
      <div class="card-content white-text">
        <i class="material-icons small left">disabled_by_default</i>
        <span class="card-title">{% blocktrans %}The invite feature is disabled.{% endblocktrans %}</span>
        <p>
          {% trans "To enable it, switch on the corresponding checkbox in the authentication section of the " %}
          <a href="{% url "preferences_site" %}">{% trans "site preferences page" %}</a>.
        </p>
      </div>
    </div>
  </div>
{% endblock %}
