{% extends "core/base.html" %}
{% load i18n %}


{% block content %}
  <div class="container">
    <div class="card red">
      <div class="card-content white-text">
        <i class="material-icons iconify small left" data-icon="mdi:alert-octagon-outline"></i>
        <span class="card-title">
        {% if exception %}
          {{ exception }}
        {% else %}
          {% trans "Error" %} (403): {% blocktrans %}You are not allowed to access the requested page or
          object.{% endblocktrans %}
        {% endif %}
        </span>
        <p>
          {% blocktrans %}
            If you think this is an error in AlekSIS, please contact your site
            administrators:
          {% endblocktrans %}
        </p>
        {% include "core/partials/admins_list.html" %}
      </div>
    </div>
  </div>
{% endblock %}
