# Generated by Django 3.1.7 on 2021-03-03 20:04

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0009_default_dashboard'),
    ]

    operations = [
        migrations.CreateModel(
            name='ExternalLinkWidget',
            fields=[
                ('dashboardwidget_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='core.dashboardwidget')),
                ('url', models.URLField(verbose_name='URL')),
                ('icon_url', models.URLField(verbose_name='Icon URL')),
            ],
            options={
                'verbose_name': 'External link widget',
                'verbose_name_plural': 'External link widgets',
            },
            bases=('core.dashboardwidget',),
        ),
    ]
