{% extends "core/base.html" %}

{% load i18n %}

{% block browser_title %}{% trans "Revoke access" %}{% endblock %}
{% block page_title %}{% trans "Revoke access" %}{% endblock %}

{% block content %}
  <div class="alert info">
    <p>
      <i class="material-icons iconify left" data-icon="mdi:alert-outline"></i>
      {% trans "Are you sure to revoke the access for this application?" %}
    </p>
  </div>

  <form method="post">
    {% csrf_token %}
    <a class="btn waves-effect waves-light red" href="{% url "oauth2_applications" %}">
      <i class="material-icons iconify left" data-icon="mdi:delete-outline"></i>
      {% trans "Revoke" %}
    </a>
    <a class="btn waves-effect waves-light" href="{% url "oauth2_applications" %}">
      <i class="material-icons iconify left" data-icon="mdi:close"></i>
      {% trans "Cancel" %}
    </a>
  </form>
{% endblock %}
