{# -*- engine:django -*- #}

{% extends "core/base.html" %}

{% load material_form i18n any_js %}

{% block extra_head %}
  {{ form.media.css }}
  {% include_css "select2-materialize" %}
{% endblock %}

{% block browser_title %}{% blocktrans %}Edit person{% endblocktrans %}{% endblock %}
{% block page_title %}{% blocktrans %}Edit person{% endblocktrans %}{% endblock %}


{% block content %}
  <form method="post" enctype="multipart/form-data">
    {% csrf_token %}
    {% form form=form %}{% endform %}
    {% include "core/partials/save_button.html" %}
  </form>
  {% include_js "select2-materialize" %}
  {{ form.media.js }}
{% endblock %}
