# Generated by Django 3.0.7 on 2020-06-28 14:02

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0003_drop_image_cropping'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='group',
            options={'ordering': ['short_name', 'name'], 'permissions': (('assign_child_groups_to_groups', 'Can assign child groups to groups'), ('view_group_stats', 'Can view statistics about group.')), 'verbose_name': 'Group', 'verbose_name_plural': 'Groups'},
        ),
        migrations.AlterField(
            model_name='group',
            name='additional_fields',
            field=models.ManyToManyField(blank=True, to='core.AdditionalField', verbose_name='Additional fields'),
        ),
    ]
