{% extends "core/base.html" %}
{% load i18n %}


{% block content %}
  <div class="container">
    <div class="card red">
      <div class="card-content white-text">
        <i class="material-icons iconify small left" data-icon="mdi:alert-octagon-outline"></i>
        <span class="card-title">{% blocktrans %}The maintenance mode is currently enabled. Please try again
          later.{% endblocktrans %}</span>
        <p>
          {% blocktrans %}
            This page is currently unavailable. If this error persists, contact your site administrators:
          {% endblocktrans %}
        </p>
        {% include "core/partials/admins_list.html" %}
      </div>
    </div>
  </div>
{% endblock %}
