{# -*- engine:django -*- #}

{% extends "core/base.html" %}

{% load i18n material_form any_js %}

{% block extra_head %}
    {{ form.media.css }}
    {% include_css "select2-materialize" %}
{% endblock %}

{% block browser_title %}
  {% if mode == "edit" %}
    {% blocktrans %}Edit announcement{% endblocktrans %}
  {% else %}
    {% blocktrans %}Publish announcement{% endblocktrans %}
  {% endif %}
{% endblock %}
{% block page_title %}
  {% if mode == "edit" %}
    {% blocktrans %}Edit announcement{% endblocktrans %}
  {% else %}
    {% blocktrans %}Publish new announcement{% endblocktrans %}
  {% endif %}
{% endblock %}

{% block content %}
  <form action="" method="post">
    {% csrf_token %}
    {% form form=form %}{% endform %}

    <button type="submit" class="btn green waves-effect waves-light">
      <i class="material-icons left iconify" data-icon="mdi:content-save-outline"></i>
      {% trans "Save and publish announcement" %}
    </button>
  </form>
  {% include_js "select2-materialize" %}
  {{ form.media.js }}
{% endblock %}
