# Generated by Django 3.2.16 on 2022-11-03 11:36

from django.db import migrations, models
import django.utils.timezone
import oauth2_provider.generators
import oauth2_provider.models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0041_update_gender_choices'),
    ]

    operations = [
        migrations.AlterField(
            model_name='pdffile',
            name='html_file',
            field=models.FileField(blank=True, null=True, upload_to='pdfs/', verbose_name='Generated HTML file'),
        ),
    ]
